package cn.fw.valhalla.domain.enums;

import com.baomidou.mybatisplus.core.enums.IEnum;
import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonValue;
import lombok.Getter;

/**
 * @author : kurisu
 * @className : DefeatReasonEnum
 * @description : 战败原因
 * @date: 2020-08-11 17:37
 */
public enum DefeatReasonEnum implements IEnum<Integer> {
    /**
     * 系统战败
     */
    SYS(1, "系统战败"),
    /**
     * 用户更换
     */
    USER(2, "用户更换"),
    /**
     * 跟进逾期战败
     */
    OVERDUE(3, "跟进逾期战败"),
    /**
     * 主动放弃
     */
    GIVE_UP(4, "主动放弃"),
    /**
     * 其他原因
     */
    OTHER(5, "其他原因")
    ;

    /**
     * 值
     */
    private final Integer value;
    /**
     * 名称
     */
    @Getter
    private final String name;

    DefeatReasonEnum(final Integer value, final String name) {
        this.value = value;
        this.name = name;
    }

    /**
     * 根据枚举值获取枚举对象
     */
    @JsonCreator
    public static DefeatReasonEnum ofValue(final Integer value) {
        for (final DefeatReasonEnum typeEnum : DefeatReasonEnum.values()) {
            if (typeEnum.value.equals(value)) {
                return typeEnum;
            }
        }
        return null;
    }

    /**
     * 获取值
     *
     * @return 值
     */
    @JsonValue
    @Override
    public Integer getValue() {
        return value;
    }

    /**
     * 获取描述
     *
     * @return 值
     */
    @JsonCreator
    public static String getNameByVale(final Integer value) {
        for (final DefeatReasonEnum typeEnum : DefeatReasonEnum.values()) {
            if (typeEnum.value.equals(value)) {
                return typeEnum.getName();
            }
        }
        return "";
    }
}
