package cn.fw.valhalla.domain.enums;

import com.baomidou.mybatisplus.core.enums.IEnum;
import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonValue;
import lombok.Getter;

/**
 * @author : kurisu
 * @className : TaskStateEnum
 * @description : 任务状态
 * @date: 2020-08-11 17:37
 */
public enum TaskStateEnum implements IEnum<Integer> {
    /**
     * 未开始
     */
    WAITING(1, "未开始"),
    /**
     * 进行中
     */
    ONGOING(2, "进行中"),
    /**
     * 已结束
     */
    END(3, "已结束");

    /**
     * 值
     */
    private final Integer value;
    /**
     * 名称
     */
    @Getter
    private final String name;

    TaskStateEnum(final Integer value, final String name) {
        this.value = value;
        this.name = name;
    }

    /**
     * 根据枚举值获取枚举对象
     */
    @JsonCreator
    public static TaskStateEnum ofValue(final Integer value) {
        for (final TaskStateEnum typeEnum : TaskStateEnum.values()) {
            if (typeEnum.value.equals(value)) {
                return typeEnum;
            }
        }
        return null;
    }

    /**
     * 获取值
     *
     * @return 值
     */
    @Override
    @JsonValue
    public Integer getValue() {
        return value;
    }

    /**
     * 获取描述
     *
     * @return 值
     */
    @JsonCreator
    public static String getNameByVale(final Integer value) {
        for (final TaskStateEnum typeEnum : TaskStateEnum.values()) {
            if (typeEnum.value.equals(value)) {
                return typeEnum.getName();
            }
        }
        return "";
    }
}
