package cn.fw.valhalla.rpc.order;

import cn.fw.data.base.domain.common.Message;
import cn.fw.order.sdk.api.CarOrderApiService;
import cn.fw.order.sdk.result.CarOrderCustomerResultDto;
import com.alibaba.fastjson.JSON;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Objects;

/**
 * fw-cas
 * <p>
 * create at 2020-05-18
 *
 * @author kurisu
 */
@Slf4j
@Service
public class OrderRpcService {

    private CarOrderApiService carOrderApiService;

    @Autowired
    OrderRpcService(CarOrderApiService carOrderApiService) {
        this.carOrderApiService = carOrderApiService;
    }

    /**
     * 查询会员在门店进行中工单关联的客户档案id
     *
     * @param memberId 会员id
     * @param shopId   门店id
     */
    public Long getOrderCustomer(Long memberId, Long shopId) {
        try {
            log.info("查询正在进行中的新车订单关联的客户档案id,memberId:{},shopId:{}", memberId, shopId);
            Message<CarOrderCustomerResultDto> msg = carOrderApiService.getMemberOrderInProgress(shopId,memberId);
            log.info("查询正在进行中的新车订单关联的客户档案id,memberId:{},shopId:{}，result:{}", JSON.toJSONString(msg));
            if (Objects.isNull(msg.getData())) {
                return null;
            }
            return msg.getData().getCustomerId();
        } catch (Exception ex) {
            log.error("查询正在进行中的新车订单关联的客户档案id失败,memberId:{},shopId:{},原因:{}", JSON.toJSONString(ex));
        }
        return null;
    }


}
