package cn.fw.valhalla.rpc.pstn;

import cn.fw.data.base.domain.common.Message;
import cn.fw.third.pstn.sdk.api.IPstnService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import static org.apache.commons.lang3.StringUtils.*;

/**
 * 电话服务
 * <p>
 * create at 2019-03-09
 *
 * @author kurisu
 */
@Slf4j
@Service
public class PstnService {

    /**
     * 智能通话服务
     */
    private final IPstnService iPstnService;

    /**
     * 默认构造器
     */
    @Autowired
    public PstnService(final IPstnService iPstnService) {
        this.iPstnService = iPstnService;
    }

    /**
     * 获取虚拟号码
     *
     * @param call   主叫电话号码
     * @param beCall 被叫电话号码
     * @return 虚拟号码
     */
    public String virtualNumber(final String call, final String beCall) {
        if (isAllBlank(call, beCall)) {
            return EMPTY;
        }
        Message<String> msg = iPstnService.callSomeone(call, beCall);
        if (!msg.isSuccess()) {
            return EMPTY;
        }
        return isBlank(msg.getData()) ? EMPTY : msg.getData();
    }
}
