package cn.fw.valhalla.sdk.enums;

import com.baomidou.mybatisplus.core.enums.IEnum;
import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonValue;
import lombok.Getter;

/**
 * @author : kurisu
 * @className : DataTypeEnum
 * @description : 数据类型
 * @date: 2020-08-11 17:37
 */
public enum DataTypeEnum implements IEnum<Integer> {
    /**
     * 订单交车
     */
    OD(1, "订单交车"),
    /**
     * 首保交车
     */
    FM(2, "首保交车"),
    /**
     * 事故车进站
     */
    AS(3, "事故车进站"),
    /**
     * 购买保险
     */
    BI(4, "购买保险"),
    /**
     * 财务结算
     */
    FS(5, "流失客户跟进"),
    ;

    /**
     * 值
     */
    private final Integer value;
    /**
     * 名称
     */
    @Getter
    private final String name;

    DataTypeEnum(final Integer value, final String name) {
        this.value = value;
        this.name = name;
    }

    /**
     * 根据枚举值获取枚举对象
     */
    @JsonCreator
    public static DataTypeEnum ofValue(final Integer value) {
        for (final DataTypeEnum typeEnum : DataTypeEnum.values()) {
            if (typeEnum.value.equals(value)) {
                return typeEnum;
            }
        }
        return null;
    }


    /**
     * 获取描述
     *
     * @return 值
     */
    public static String getNameByVale(final Integer value) {
        for (final DataTypeEnum typeEnum : DataTypeEnum.values()) {
            if (typeEnum.value.equals(value)) {
                return typeEnum.getName();
            }
        }
        return "";
    }

    @Override
    @JsonValue
    public Integer getValue() {
        return value;
    }
}
