package cn.fw.valhalla.sdk.param;

import cn.fw.valhalla.sdk.enums.CarUseTypeEnum;
import cn.fw.valhalla.sdk.enums.CusTypeEnum;
import cn.fw.valhalla.sdk.enums.GenderEnum;
import cn.fw.valhalla.sdk.enums.SourceTypeEnum;
import lombok.Data;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.math.BigDecimal;
import java.util.Date;

/**
 * 保有客信息
 *
 * @author Kurisu
 */
@Data
public class CustomerParams {
    /**
     * 会员id
     */
    @NotNull(message = "会员id不能为空")
    private Long memberId;
    /**
     * 名称
     */
    private String name;
    /**
     * 客户联系电话
     */
    @NotBlank(message = "客户联系电话不能为空")
    private String mobile;
    /**
     * 客户性别
     *
     */
    private GenderEnum gender;
    /**
     * 客户出生日期
     */
    private Date birthday;
    /**
     * 客户证件号码 或者单位的组织机构代码
     */
    @NotBlank(message = "证件号码不能为空")
    private String idCode;
    /**
     * 客户类型;1:个人；2:单位
     *
     */
    private CusTypeEnum cusType;
    /**
     * 工作单位 （没有的情况默认个人）
     * 如果是公司客户，此处填写公司名称
     */
    private String companyName = "个人";
    /**
     * 联系地址
     */
    private String address;
    /**
     * 纬度
     */
    private BigDecimal lat;
    /**
     * 经度
     */
    private BigDecimal lng;
    /**
     * 车牌号
     */
    @NotBlank(message = "车牌号不能为空")
    private String plateNo;
    /**
     * 车架号
     */
    private String frameNo;
    /**
     * 发动机编号
     */
    private String engineNo;
    /**
     * 使用地城市代码
     */
    private String cityCode;
    /**
     * 使用地城市名称
     */
    private String cityName;
    /**
     * 行驶证注册日期
     */
    private Date regDate;
    /**
     * 车型代码
     */
    private String specCode;
    /**
     * 品牌id
     */
    @NotNull(message = "品牌id不能为空")
    private Long brandId;
    /**
     * 品牌名称
     */
    private String brandName;
    /**
     * 车系id
     */
    @NotNull(message = "车系id不能为空")
    private Long seriesId;
    /**
     * 车系名称
     */
    private String seriesName;
    /**
     * 车系id
     */
    @NotNull(message = "车型id不能为空")
    private Long specId;
    /**
     * 车系名称
     */
    private String specName;
    /**
     * 行驶证fid
     */
    private String vehicleLicense;
    /**
     * 车辆图片
     */
    private String carImage;
    /**
     * 当前行驶里程 单位KM
     */
    private Integer currentMileage;
    /**
     * 购车时间/开票时间
     */
    private Date buyDate;
    /**
     * 购买价格/开票价格
     */
    private Long buyPrice;
    /**
     * 使用类型
     *
     */
    @NotNull(message = "使用类型不能为空")
    private CarUseTypeEnum useType;
    /**
     * 来源;
     *
     */
    @NotNull(message = "客户来源不能为空")
    private SourceTypeEnum source;
    /**
     * 所属服务站id
     */
    @NotNull(message = "所属服务站ID不能为空")
    private Long shopId;
    /**
     * 所属商家id
     */
    private Long dealerId;
    /**
     * 所属集团
     */
    @NotNull(message = "所属集团ID不能为空")
    private Long groupId;

    /**
     * 服务顾问id
     */
    @NotNull(message = "服务顾问id不能为空")
    private Long adviserId;
}
