package cn.fw.valhalla.service.bus;

import cn.fw.valhalla.common.constant.RoleCode;
import cn.fw.valhalla.domain.enums.FollowTypeEnum;
import cn.fw.valhalla.domain.vo.PostUserVO;
import cn.fw.valhalla.rpc.erp.UserService;
import cn.fw.valhalla.rpc.erp.dto.PostUserDTO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.stream.Collectors;

import static cn.fw.common.businessvalidator.Validator.BV;

/**
 * @author : kurisu
 * @className : CommonService
 * @description : 公共服务
 * @date: 2020-10-22 11:07
 */
@Service
@Slf4j
public class CommonService {
    private final UserService userService;

    @Autowired
    public CommonService(final UserService userService) {
        this.userService = userService;
    }

    public List<PostUserVO> getUsers(final Long shopId, final Integer type) {
        FollowTypeEnum typeEnum = FollowTypeEnum.ofValue(type);
        BV.notNull(typeEnum, () -> "跟进类型错误");
        List<PostUserDTO> userByRole = userService.getUserByRole(shopId, getRoleCode(typeEnum));
        return userByRole.stream().map(user -> new PostUserVO(user.getUserId(), user.getUserName())).collect(Collectors.toList());
    }

    private String getRoleCode(FollowTypeEnum followTypeEnum) {
        switch (followTypeEnum) {
            case AC:
                return RoleCode.SGCGJ;
            case IR:
                return RoleCode.XBGJ;
            case FM:
            case RM:
            default:
                return RoleCode.FWGW;
        }
    }
}
